#ifndef GEOMETRYMAKER_H
#define GEOMETRYMAKER_H

#include <cmath>

#include "cvec.h"

//--------------------------------------------------------------------------------
// Helpers for creating some special geometries such as plane, cubes, and spheres
//--------------------------------------------------------------------------------


// A generic vertex structure containing position, normal, and texture information
// Used by make* functions to pass vertex information to the caller
struct GenericVertex {
  Cvec3f pos;
  Cvec3f normal;
  Cvec2f tex;
  Cvec3f tangent, binormal;

  GenericVertex() {}
  GenericVertex(
    float x, float y, float z,
    float nx, float ny, float nz,
    float tu, float tv,
    float tx, float ty, float tz,
    float bx, float by, float bz)
    : pos(x,y,z), normal(nx,ny,nz), tex(tu, tv), tangent(tx, ty, tz), binormal(bx, by, bz)
  {}
    GenericVertex(const GenericVertex& v) {
        *this = v;
    }

    GenericVertex& operator = (const GenericVertex& v) {
        pos = v.pos;
        normal = v.normal;
        tex = v.tex;
        tangent = v.tangent;
        binormal = v.binormal;
        return *this;
    }
};

inline void getPlaneVbIbLen(int& vbLen, int& ibLen) {
  vbLen = 4;
  ibLen = 6;
}

template<typename VtxOutIter, typename IdxOutIter>
void makePlane(float size, VtxOutIter vtxIter, IdxOutIter idxIter) {
#define DEFV(x, y, z, nx, ny, nz, tu, tv) { \
	*vtxIter = GenericVertex(x h, y h, z h, \
	nx, ny, nz, tu, tv, \
	tan[0], tan[1], tan[2], \
	bin[0], bin[1], bin[2]); \
	++vtxIter; \
}
  float h = size / 2.0;
  Cvec3f tan(1, 0, 0), bin(0, 1, 0);
  DEFV(-, -, +, 0, 0, 1, 0, 0); // facing +Z
  DEFV(+, -, +, 0, 0, 1, 1, 0);
  DEFV(+, +, +, 0, 0, 1, 1, 1);
  DEFV(-, +, +, 0, 0, 1, 0, 1);
#undef DEFV

  for (int v = 0; v < 4; v += 4) {
	  *idxIter = v;
	  *++idxIter = v + 1;
	  *++idxIter = v + 2;
	  *++idxIter = v;
	  *++idxIter = v + 2;
	  *++idxIter = v + 3;
	  ++idxIter;
  }
}


#endif
